﻿using System;
using NiVE2.Plugin.Interface;
using NiVE2.Plugin.Utils;
using NiVE2.Utils;

namespace ruche.nive2.effects
{
    /// <summary>
    /// 整数値の配列を格納するNiVE2プロパティクラス。
    /// </summary>
    [Serializable]
    public class IntArrayProperty : PropertyBase
    {
        /// <summary>
        /// サポート可能な補間タイプ。
        /// </summary>
        public const PropertyInterpolationType SupportableInterpolationTypes =
            PropertyInterpolationType.Fixed |
            PropertyInterpolationType.Liner;

        /// <summary>
        /// 整数値配列。
        /// </summary>
        private int[] _values = new int[] { 0 };

        /// <summary>
        /// 既定値。
        /// </summary>
        private int _defaultValue = 0;

        /// <summary>
        /// 最大値。
        /// </summary>
        private int _maxValue = int.MaxValue;

        /// <summary>
        /// 最小値。
        /// </summary>
        private int _minValue = int.MinValue;

        /// <summary>
        /// サポートする補間タイプ。
        /// </summary>
        private PropertyInterpolationType _interpolationTypes =
            SupportableInterpolationTypes;

        /// <summary>
        /// カンマ区切りの文字列値から整数値配列を作成する。
        /// </summary>
        /// <param name="value">カンマ区切りの文字列値。</param>
        /// <param name="defaultValue">既定値。</param>
        /// <returns>整数値配列。</returns>
        public static int[] ParseToIntArray(string value, int defaultValue)
        {
            // null もしくは空文字列ならば既定値1個の配列を返す
            value = (value == null) ? "" : value.Trim();
            if (value == "" || value == ",")
            {
                return new int[] { defaultValue };
            }

            // 末尾がカンマなら取り除く
            if (value.EndsWith(","))
            {
                value = value.Substring(0, value.Length - 1);
            }

            // カンマで区切る
            string[] numStrs = value.Split(',');

            // それぞれ数値にする
            int[] values = new int[numStrs.Length];
            for (int i = 0; i < values.Length; ++i)
            {
                try
                {
                    values[i] = int.Parse(numStrs[i]);
                }
                catch
                {
                    values[i] = defaultValue;
                }
            }

            return values;
        }

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        /// <param name="name">プロパティ名。</param>
        /// <param name="values">既定の整数値配列。</param>
        /// <param name="defaultValue">既定値。</param>
        /// <param name="maxValue">最大値。</param>
        /// <param name="minValue">最小値。</param>
        public IntArrayProperty(
            string name,
            int[] values,
            int defaultValue,
            int maxValue,
            int minValue)
            :
            base(name)
        {
            _maxValue = maxValue;
            _minValue = Math.Min(minValue, maxValue);
            DefaultValue = defaultValue;
            Values = values;
        }

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        /// <param name="name">プロパティ名。</param>
        /// <param name="values">既定の整数値配列。</param>
        /// <param name="defaultValue">既定値。</param>
        public IntArrayProperty(
            string name,
            int[] values,
            int defaultValue)
            :
            this(name, values, defaultValue, int.MaxValue, int.MinValue)
        {
        }

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        /// <param name="name">プロパティ名。</param>
        /// <param name="values">既定の整数値配列。</param>
        /// <param name="defaultValue">既定値。</param>
        /// <param name="maxValue">最大値。</param>
        /// <param name="minValue">最小値。</param>
        /// <param name="interpolationTypes">サポートする補間タイプ。</param>
        /// <remarks>
        /// 現在サポート可能な補間タイプは固定と線形のみ。
        /// </remarks>
        public IntArrayProperty(
            string name,
            int[] values,
            int defaultValue,
            int maxValue,
            int minValue,
            PropertyInterpolationType interpolationTypes)
            :
            this(name, values, defaultValue, maxValue, minValue)
        {
            _interpolationTypes =
                interpolationTypes & SupportableInterpolationTypes;
        }

        /// <summary>
        /// コピーコンストラクタ。
        /// </summary>
        /// <param name="src">コピー元。</param>
        public IntArrayProperty(IntArrayProperty src)
            :
            this(
                src.PropertyName,
                src.Values,
                src.DefaultValue,
                src.Max,
                src.Min,
                src.SupportInterpolationType)
        {
        }

        /// <summary>
        /// 整数値配列を取得または設定する。
        /// </summary>
        public int[] Values
        {
            get { return _values; }
            set
            {
                _values = (value == null) ?
                    (new int[] { DefaultValue }) : FixValues(value);
            }
        }

        /// <summary>
        /// 整数値配列の要素数を取得または設定する。
        /// </summary>
        public int Count
        {
            get { return Values.Length; }
            set
            {
                if (value == Values.Length) { return; }

                int[] dest = new int[value];
                if (value < Values.Length)
                {
                    Array.Copy(Values, dest, value);
                }
                else if (value > Values.Length)
                {
                    Array.Copy(Values, dest, Values.Length);
                    for (int i = Values.Length; i < value; ++i)
                    {
                        dest[i] = DefaultValue;
                    }
                }
                Values = dest;
            }
        }

        /// <summary>
        /// 整数値配列要素を取得または設定するインデクサ。
        /// </summary>
        /// <param name="index">配列要素のインデックス。</param>
        /// <returns>整数値配列要素。</returns>
        public int this[int index]
        {
            get { return Values[index]; }
            set { Values[index] = FixValue(value); }
        }

        /// <summary>
        /// 既定値を取得または設定する。
        /// </summary>
        public int DefaultValue
        {
            get { return _defaultValue; }
            set { _defaultValue = FixValue(value); }
        }

        /// <summary>
        /// 最大値を取得または設定する。
        /// </summary>
        public int Max
        {
            get { return _maxValue; }
            set
            {
                // 最大値設定
                _maxValue = value;
                if (_minValue > value)
                {
                    _minValue = value;
                }

                // 値を補正
                DefaultValue = FixValue(DefaultValue);
                Values = FixValues(Values);
            }
        }

        /// <summary>
        /// 最小値を取得または設定する。
        /// </summary>
        public int Min
        {
            get { return _minValue; }
            set
            {
                // 最小値設定
                _minValue = value;
                if (_maxValue < value)
                {
                    _maxValue = value;
                }

                // 値を補正
                DefaultValue = FixValue(DefaultValue);
                Values = FixValues(Values);
            }
        }

        /// <summary>
        /// 整数値を最小値および最大値で補正する。
        /// </summary>
        /// <param name="value">整数値。</param>
        /// <returns>補正された整数値。</returns>
        private int FixValue(int value)
        {
            return Math.Min(Math.Max(value, _minValue), _maxValue);
        }

        /// <summary>
        /// 整数値配列を最小値および最大値で補正する。
        /// </summary>
        /// <param name="values">整数値配列。</param>
        /// <returns>補正された整数値配列。</returns>
        /// <remarks>
        /// 引数の配列自体は変更しない。引数が null の場合は null を返す。
        /// </remarks>
        private int[] FixValues(int[] values)
        {
            if (values == null) { return null; }

            int[] vals = new int[values.Length];
            for (int i = 0; i < vals.Length; ++i)
            {
                vals[i] = FixValue(values[i]);
            }
            return vals;
        }

        /// <summary>
        /// 整数値配列をカンマ区切りの文字列値に変換して取得する。
        /// </summary>
        /// <returns>カンマ区切りの文字列値。</returns>
        public string GetCsvString()
        {
            string result = "";

            for (int i = 0; i < Values.Length; ++i)
            {
                result += Values[i].ToString();
                if (i < Values.Length - 1)
                {
                    result += ",";
                }
            }

            return result;
        }

        /// <summary>
        /// カンマ区切りの文字列値から整数値配列を設定する。
        /// </summary>
        /// <param name="value">カンマ区切りの文字列値。</param>
        public void SetCsvString(string value)
        {
            Values = ParseToIntArray(value, DefaultValue);
        }

        public override string ToString()
        {
            return GetCsvString();
        }

        #region PropertyBase メンバ

        public override object Value
        {
            get { return _values; }
            set { _values = FixValues((int[])value); }
        }

        public override PropertyInterpolationType SupportInterpolationType
        {
            get { return _interpolationTypes; }
        }

        public override PropertyBase Interpolation(
            KeyFrame[] keyFrame, double time)
        {
            // 取得時刻以前＆超で最も近いキーフレームを探す
            KeyFrame beforeKey = null;
            KeyFrame afterKey = null;
            foreach (KeyFrame key in keyFrame)
            {
                if (key.Time <= time)
                {
                    if (beforeKey == null || key.Time > beforeKey.Time)
                    {
                        beforeKey = key;
                    }
                }
                else
                {
                    if (afterKey == null || key.Time < afterKey.Time)
                    {
                        afterKey = key;
                    }
                }
            }

            // プロパティ値作成
            IntArrayProperty property = null;
            if (beforeKey == null)
            {
                property = (afterKey == null) ?
                    (new IntArrayProperty(this)) :
                    (IntArrayProperty)afterKey.Property.Copy();
            }
            else
            {
                property = (IntArrayProperty)beforeKey.Property.Copy();

                // 線形補間
                if (
                    beforeKey.Type == PropertyInterpolationType.Liner &&
                    afterKey != null)
                {
                    IntArrayProperty after =
                        (IntArrayProperty)afterKey.Property;
                    int count = Math.Min(property.Count, after.Count);
                    for (int i = 0; i < count; ++i)
                    {
                        property[i] =
                            ValueInterpolationMethod.Linear(
                                beforeKey.Time,
                                property[i],
                                afterKey.Time,
                                after[i],
                                time);
                    }
                }
            }

            return property;
        }

        public override PropertyBase Copy()
        {
            return new IntArrayProperty(this);
        }

        public override PropertyBase PasteProperty(PropertyBase property)
        {
            return property;
        }

        public override PropertyBase[] PasteProperty(PropertyBase[] property)
        {
            return property;
        }

        public override bool Equals(PropertyBase obj)
        {
            if (obj is IntArrayProperty)
            {
                int[] vals = ((IntArrayProperty)obj).Values;
                if (Values.Length == vals.Length)
                {
                    for (int i = 0; i < vals.Length; ++i)
                    {
                        if (Values[i] != vals[i])
                        {
                            return false;
                        }
                    }
                    return true;
                }
            }
            return false;
        }

        #endregion
    }
}
